import bpy


def write_constraints_copy_rotation(f, constraint):
    f.write("  (constraint 'copy_rotation'\n")
    f.write("    influence: {}\n".format(constraint.influence))
    f.write("    target: {}\n".format(constraint.target.name))
    f.write("    subtarget: {}\n".format(constraint.subtarget))
    f.write("    owner_space: {}\n".format(constraint.owner_space))
    f.write("    target_space: {}\n".format(constraint.target_space))
    f.write("    use_xyz: [{} {} {}]\n".format(
        constraint.use_x,
        constraint.use_y,
        constraint.use_z,
    ))
    f.write("    invert_xyz: [{} {} {}]\n".format(
        constraint.invert_x,
        constraint.invert_y,
        constraint.invert_z,
    ))
    f.write("    mix_mode: {}\n".format(constraint.mix_mode))
    f.write("    euler_order: {}\n".format(constraint.euler_order))
    f.write("    )")


def write_constraint_limit_rotation(f, constraint):
    f.write("  (constraint 'limit_rotation'\n")
    f.write("    influence: {}\n".format(constraint.influence))
    f.write("    owner_space: {}\n".format(constraint.owner_space))
    f.write("    use_limit_xyz: [{} {} {}]\n".format(
        constraint.use_limit_x,
        constraint.use_limit_y,
        constraint.use_limit_z,
    ))
    f.write("    min_xyz: [{} {} {}]\n".format(
        constraint.min_x,
        constraint.min_y,
        constraint.min_z,
    ))
    f.write("    max_xyz: [{} {} {}]\n".format(
        constraint.max_x,
        constraint.max_y,
        constraint.max_z,
    ))


def write_bone_constraints(context, filepath):
    # Get the active object
    obj = context.active_object

    # Open the file for writing
    with open(filepath, 'w') as file:
        # Iterate over all bones
        for bone in obj.pose.bones:
            # Write bone name
            file.write("(bone '{}'".format(bone.name))

            # Iterate over all bone constraints
            for constraint in bone.constraints:
                if not constraint.enabled or constraint.mute or not constraint.active:
                    continue

                # Write constraint type and settings
                file.write("\n")

                if isinstance(constraint, bpy.types.CopyRotationConstraint):
                    write_constraints_copy_rotation(file, constraint)
                elif isinstance(constraint, bpy.types.LimitRotationConstraint):
                    write_constraint_limit_rotation(file, constraint)
                else:
                    # Unknown constraint type
                    # Print python class name
                    file.write("; unknown constraint.\n; class: {}\n".format(
                        constraint.__class__.__name__))
                    file.write(";  (constraint '{}'\n".format(constraint.type))
                    for prop in dir(constraint):
                        if not prop.startswith('_') and not callable(
                                getattr(constraint, prop)):
                            file.write(";    {}: {}\n".format(
                                prop, getattr(constraint, prop)))

                    file.write(";  )\n")
            file.write(")\n")


class ExportConstraint(bpy.types.Operator):
    bl_idname = "solstice.export_bone_constraints"
    bl_label = "Export Bone Constraints"

    filepath: bpy.props.StringProperty()

    def execute(self, context):
        # Assert the selected file is a bone
        if context.active_object.type != 'ARMATURE':
            self.report({'ERROR'}, "Active object is not an armature.")
            return {'CANCELLED'}

        # Do something with the selected file path
        print("Selected file:", self.filepath)
        # Call the function or perform the desired actions using the file path

        write_bone_constraints(context, self.filepath)

        return {'FINISHED'}

    def invoke(self, context, event):
        # Open the file browser dialog
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}
